/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.pdfconv;

import de.hendriklipka.pdfconv.config.ImageConfig;
import org.jpedal.PdfDecoder;

import java.awt.geom.AffineTransform;
import java.awt.image.*;

public class ImageProcessor
{
    private ImageConfig _config;

    private LogHandler _logger;

    public ImageProcessor(ImageConfig config)
    {
        _config=config;
    }

    public BufferedImage processImage(BufferedImage image)
    {
        BufferedImage targetImage=null;

            _logger.printDebug("image: width="+image.getWidth()+", heigth="+image.getHeight());

        String version=PdfDecoder.version;
        double d=Double.parseDouble(version);
        if (d>1.81) // flipping the image is needed since jpedal version 1.81
        {
            _logger.printDebug("flipping image");
            AffineTransform at=new AffineTransform();
            at.scale(1, -1); // flip image upside-down
            at.translate(0, -image.getHeight());
            _logger.printDebug("doing transformation");
            AffineTransformOp ato=new AffineTransformOp(at, null);
            targetImage=ato.filter(image, null);
            image=targetImage;
        }

        double scaleFactor=_config.scaleFactor;
        double heigth=image.getHeight();
        double width=image.getWidth();
        if (-1!=_config.maxHeigth||-1!=_config.maxWidth||1!=_config.scaleFactor)
        {
            boolean scaleImage=false;
            _logger.printDebug("checking for scale");
            if (-1!=_config.maxWidth&&_config.maxWidth<width)
            {
                d=_config.maxWidth/width;
                if (d<scaleFactor||-1==scaleFactor)
                {
                    scaleFactor=d;
                }
                scaleImage=true;
                _logger.printDebug("scale width="+scaleFactor);
            }
            if (-1!=_config.maxHeigth&&_config.maxHeigth<heigth)
            {
                d=_config.maxHeigth/heigth;
                _logger.printDebug("scale heigth="+d);
                if (d<scaleFactor||-1==scaleFactor)
                {
                    scaleFactor=d;
                }
                scaleImage=true;
            }
            if (1!=_config.scaleFactor)
                scaleImage=true;
            if (scaleImage)
            {
                _logger.printDebug("scale factor="+scaleFactor);
                AffineTransform at=new AffineTransform();
                at.scale(scaleFactor, scaleFactor);
                _logger.printDebug("doing transformation");
                AffineTransformOp ato=new AffineTransformOp(at, null);
                targetImage=ato.filter(image, null);
                _logger.printDebug("target image: width="+targetImage.getWidth()+", heigth="+targetImage.getHeight());
                image=targetImage;
            }
        }

        return targetImage;
    }

    public void setLogger(LogHandler logger)
    {
        _logger=logger;
    }
}
