package de.hendriklipka.pdfconv.gui;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.PanelBuilder;

import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class FileTypesPage extends BaseWizardPage
{
    JCheckBox _cbHTML=null;
    JCheckBox _cbBookWin=null;
    JCheckBox _cbBookMac=null;
    JCheckBox _cbNTKWin=null;
    JCheckBox _cbNTKMac=null;
    private boolean _allowed=false;
    private ActionListener _listener=new StateListener();

    public FileTypesPage(LogicHandler logicHandler)
    {
        _logicHandler=logicHandler;
        _enabled=true;
    }

    protected void buildPanel()
    {
        FormLayout layout=new FormLayout("fill:100dlu,10dlu,fill:default:grow",
            "d,3,d,3,d,3,d,3,d,3,d,fill:0dlu:grow");

        PanelBuilder builder=new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        _panel=builder.getPanel();
        _panel.putClientProperty("jgoodies.noContentBorder", Boolean.TRUE);
        _panel.setBorder(BorderFactory.createEmptyBorder());

        CellConstraints cc=new CellConstraints();

        JTextArea jt=new JTextArea("Specify which container files you want to create.");
        jt.setEditable(false);
        jt.setEnabled(true);
        jt.setWrapStyleWord(true);
        jt.setLineWrap(true);
        builder.add(jt, cc.xywh(1, 1, 1, 12));

        builder.addTitle("What files do you want to generate?", cc.xywh(3, 1, 1, 1));

        _cbHTML=new JCheckBox("HTML file");
        _cbHTML.setFocusPainted(false);
        builder.add(_cbHTML, cc.xy(3, 3));
        _cbBookWin=new JCheckBox("BookMaker text file for Windows");
        _cbBookWin.setFocusPainted(false);
        builder.add(_cbBookWin, cc.xy(3, 5));
        _cbBookMac=new JCheckBox("BookMaker text file for Macintosh");
        _cbBookMac.setFocusPainted(false);
        builder.add(_cbBookMac, cc.xy(3, 7));
        _cbNTKWin=new JCheckBox("NTK book project for Windows");
        _cbNTKWin.setFocusPainted(false);
        builder.add(_cbNTKWin, cc.xy(3, 9));
        _cbNTKMac=new JCheckBox("NTK book project for Macintosh");
        _cbNTKMac.setFocusPainted(false);
        builder.add(_cbNTKMac, cc.xy(3, 11));
        
        _cbBookMac.addActionListener(_listener);
        _cbBookWin.addActionListener(_listener);
        _cbNTKMac.addActionListener(_listener);
        _cbNTKWin.addActionListener(_listener);
        _cbHTML.addActionListener(_listener);
    }

    public boolean allowFinish()
    {
        return _allowed;
    }

    public void leavePage()
    {
        _logicHandler.setConfigurationData(PageConstants.FILE_TYPE_HTML,_cbHTML.isSelected()?"true":"false");
        _logicHandler.setConfigurationData(PageConstants.FILE_TYPE_BOOK_MAC, _cbBookMac.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.FILE_TYPE_BOOK_WIN, _cbBookWin.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.FILE_TYPE_NTK_MAC, _cbNTKMac.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.FILE_TYPE_NTK_WIN, _cbNTKWin.isSelected() ? "true" : "false");
    }

    public void setDesiredFileType(String type)
    {
        _cbBookMac.setSelected(false);
        _cbBookWin.setSelected(false);
        _cbNTKMac.setSelected(false);
        _cbNTKWin.setSelected(false);
        _cbHTML.setSelected(false);
        if (type.equalsIgnoreCase(PageConstants.FILE_TYPE_BOOK_MAC))
        {
            _cbBookMac.setSelected(true);
        }
        else if (type.equalsIgnoreCase(PageConstants.FILE_TYPE_BOOK_WIN))
        {
            _cbBookWin.setSelected(true);
        }
        else if (type.equalsIgnoreCase(PageConstants.FILE_TYPE_NTK_MAC))
        {
            _cbNTKMac.setSelected(true);
        }
        else if (type.equalsIgnoreCase(PageConstants.FILE_TYPE_NTK_WIN))
        {
            _cbNTKWin.setSelected(true);
        }
        else if (type.equalsIgnoreCase(PageConstants.FILE_TYPE_HTML))
        {
            _cbHTML.setSelected(true);
        }
    }

    class StateListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            if (_cbBookMac.isSelected())
                _allowed=true;
            else if (_cbBookWin.isSelected())
                _allowed=true;
            else if (_cbNTKMac.isSelected())
                _allowed=true;
            else if (_cbNTKWin.isSelected())
                _allowed=true;
            else if (_cbHTML.isSelected())
                _allowed=true;
            else
                _allowed=false;
            _logicHandler.changeFinishState();
        }
    }
}
