package de.hendriklipka.pdfconv.gui;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.PanelBuilder;

import javax.swing.*;
import javax.swing.event.CaretListener;
import javax.swing.event.CaretEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

public class OutputOptionsPage extends BaseWizardPage
{
    private JLabel _msgLabel=null;
    private JCheckBox _checkboxMessages=null;
    private JCheckBox _checkboxVerbose = null;
    private JCheckBox _checkboxDebug = null;
    private JTextField _baseName=null;
    private JTextField _targetDir=null;
    private JButton _browseDir=null;
    private JTextField _pageSpec=null;
    private boolean _allowed=false;

    public OutputOptionsPage(LogicHandler logicHandler)
    {
        _logicHandler=logicHandler;
        _enabled=true;
    }

    protected void buildPanel()
    {
        FormLayout layout=new FormLayout("fill:100dlu,10dlu,right:max(40dlu;pref),3dlu,fill:default:grow,3dlu,pref",
            "p,3,p,3,p,10,p,10,p,3,p,3,p,fill:pref:grow");

        PanelBuilder builder=new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        _panel=builder.getPanel();
        int[][] rowGroups={{1,3,5}};
        layout.setRowGroups(rowGroups);
        _panel.putClientProperty("jgoodies.noContentBorder", Boolean.TRUE);
        _panel.setBorder(BorderFactory.createEmptyBorder());

        CellConstraints cc=new CellConstraints();

        JTextArea jt=new JTextArea("Specify how to generate the output.\n\nAdditionally, you can specify the page range(s) to include in the output.");
        jt.setEditable(false);
        jt.setEnabled(true);
        jt.setWrapStyleWord(true);
        jt.setLineWrap(true);
        builder.add(jt, cc.xywh(1, 1, 1, 14));

        builder.addTitle("How do you want to store the results?", cc.xywh(3, 1, 5, 1));

        _baseName=new JTextField();
        builder.add(_baseName, cc.xy(5, 3));
        builder.addLabel("Basename", cc.xy(3, 3));
        _baseName.addCaretListener(new CaretListener()
        {
            public void caretUpdate(CaretEvent e)
            {
                setStates();
            }
        });

        _targetDir=new JTextField();
        builder.add(_targetDir, cc.xy(5, 5));
        builder.addLabel("Target dir", cc.xy(3, 5));
        _browseDir=new JButton("Browse...");
        builder.add(_browseDir, cc.xy(7, 5));
        _targetDir.addCaretListener(new CaretListener()
        {
            public void caretUpdate(CaretEvent e)
            {
                setStates();
            }
        });

        builder.addLabel("Pages",cc.xy(3,7));
        _pageSpec=new JTextField();
        builder.add(_pageSpec,cc.xy(5,7));

        _msgLabel=builder.addLabel("Messages", cc.xy(3, 9));

        _checkboxMessages=new JCheckBox("Show messages");
        _checkboxMessages.setFocusPainted(false);
        builder.add(_checkboxMessages, cc.xy(5, 9));
        
        _checkboxVerbose = new JCheckBox("Show verbose messages");
        _checkboxVerbose.setFocusPainted(false);
        builder.add(_checkboxVerbose, cc.xy(5, 11));
        _checkboxVerbose.setEnabled(false);

        _checkboxDebug = new JCheckBox("Show debug messages");
        _checkboxDebug.setFocusPainted(false);
        builder.add(_checkboxDebug, cc.xy(5, 13));
        _checkboxDebug.setEnabled(false);

        _browseDir.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                browseForTargetDir();
            }
        });
        _checkboxMessages.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                _checkboxVerbose.setEnabled(_checkboxMessages.isSelected());
                if (_checkboxVerbose.isEnabled())
                    _checkboxDebug.setEnabled(_checkboxVerbose.isSelected());
                else
                    _checkboxDebug.setEnabled(false);
            }
        });
        _checkboxVerbose.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                _checkboxDebug.setEnabled(_checkboxVerbose.isSelected());
            }
        });
    }

    private void browseForTargetDir()
    {
        String currentDirName=_targetDir.getText();
        File currentDir=new File(currentDirName);
        if (!currentDir.exists())
            currentDir=new File(".");
        JFileChooser chooser=new JFileChooser(currentDir);    
        chooser.setDialogTitle("Select PDF source file");
        chooser.setMultiSelectionEnabled(false);
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        int retVal=chooser.showOpenDialog(this._panel);
        try
        {
            if (retVal==JFileChooser.APPROVE_OPTION)
            {
                currentDirName=chooser.getSelectedFile().getCanonicalPath();
            }
        }
        catch (IOException e)
        {
        }
        _targetDir.setText(currentDirName);
        setStates();
    }

    public void leavePage()
    {
        _logicHandler.setConfigurationData(PageConstants.SHOW_MESSAGES, _checkboxMessages.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.SHOW_VERBOSE, _checkboxVerbose.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.SHOW_DEBUG, _checkboxDebug.isSelected() ? "true" : "false");
        _logicHandler.setConfigurationData(PageConstants.FILE_BASENAME, _baseName.getText());
        _logicHandler.setConfigurationData(PageConstants.FILE_OUTDIR, _targetDir.getText());
        _logicHandler.setConfigurationData(PageConstants.PAGES, _pageSpec.getText());
    }

    public boolean allowFinish()
    {
        return _allowed;
    }

    public void setAdvancedOptions(boolean advanced)
    {
        if (advanced)
        {
            _checkboxMessages.setVisible(true);
            _checkboxVerbose.setVisible(true);
            _checkboxDebug.setVisible(true);
            _msgLabel.setVisible(true);
        }
        else
        {
            _checkboxMessages.setVisible(false);
            _checkboxVerbose.setVisible(false);
            _checkboxDebug.setVisible(false);
            _msgLabel.setVisible(false);
        }
    }
    
    private void setStates()
    {
        if (0!=_targetDir.getText().length())
        {
            _allowed=true;
        }
        else
        {
            _allowed=false;
        }
        _logicHandler.changeFinishState();
    }

    public String getTargetDirectory()
    {
        return _targetDir.getText();
    }

    public String getBaseName()
    {
        return _baseName.getText();
    }

    public boolean isVerbose()
    {
        return _checkboxMessages.isSelected();
    }
}
