package de.hendriklipka.pdfconv.gui;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.builder.PanelBuilder;

import javax.swing.*;
import javax.swing.text.PlainDocument;

import de.hendriklipka.pdfconv.LogHandler;

import java.io.File;

public class ShowProgressPage extends BaseWizardPage implements LogHandler
{
    private JLabel _fileName=null;
    private JProgressBar _fileProgress=null;
    private JLabel _page=null;
    private JProgressBar _pageProgress=null;
    private JTextArea _messages=null;
    
    private int _fileCount=1;
    private int _pageCount=1;
    
    private PlainDocument _messageData=null;
    private boolean _showMessages=false;
    private boolean _verbose=false;
    private boolean _debug=false;
    private JLabel _messagesLabel;
    private JScrollPane _messagesPane;

    public ShowProgressPage(LogicHandler handler)
    {
        _logicHandler=handler;
        _enabled=false;
    }

    protected void buildPanel()
    {
        FormLayout layout=new FormLayout("fill:100dlu,10dlu,pref,3dlu,fill:default:grow",
            "d,3,d,3,d,3,d,3,d,3,d,3,fill:0dlu:grow");

        PanelBuilder builder=new PanelBuilder(layout);
        builder.setDefaultDialogBorder();

        _panel=builder.getPanel();
        _panel.putClientProperty("jgoodies.noContentBorder", Boolean.TRUE);
        _panel.setBorder(BorderFactory.createEmptyBorder());

        CellConstraints cc=new CellConstraints();

        JTextArea jt=new JTextArea("This page shows the progress of the conversion process.");
        jt.setEditable(false);
        jt.setEnabled(true);
        jt.setWrapStyleWord(true);
        jt.setLineWrap(true);
        builder.add(jt, cc.xywh(1, 1, 1, 13));

        builder.addTitle("Generating output", cc.xywh(3, 1, 3, 1));

        builder.addLabel("File(s):",cc.xy(3,3));
        _fileName=new JLabel();
        builder.add(_fileName,cc.xy(5,3));
        _fileProgress=new JProgressBar();
        builder.add(_fileProgress,cc.xywh(3,5,3,1));
        _fileProgress.setStringPainted(true);
        

        builder.addLabel("Page(s):",cc.xy(3,7));
        _pageProgress=new JProgressBar();
        builder.add(_pageProgress,cc.xywh(3,9,3,1));
        _pageProgress.setStringPainted(true);

        _messageData=new PlainDocument();
        _messagesLabel=new JLabel("Messages");
        builder.add(_messagesLabel,cc.xy(3,11));
        _messages=new JTextArea(_messageData);
        _messages.setEditable(false);
        _messages.setLineWrap(false);
        _messages.setDoubleBuffered(true);
        _messagesPane= new JScrollPane(_messages,
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        builder.add(_messagesPane,cc.xywh(3,13,3,1));
    }
    
    public boolean allowFinish()
    {
        return false;
    }

    public void leavePage()
    {
    }

    public void setVerbose(boolean verbose)
    {
        _verbose=verbose;
    }

    public void setDebug(boolean debug)
    {
        _debug=debug;
    }

    public void setFileName(String fileName)
    {
        File f=new File(fileName);
        
        String name=fileName;
        if (f.exists())
            name=f.getName();
        updateLabel(_fileName, name);
    }

    public void setFileCount(int files)
    {
        _fileCount=files;
        initProgressBar(_fileProgress, files);
    }

    public void setCurrentFile(int file)
    {
        updateProgressBar(file+1, _fileCount, _fileProgress,null);
    }

    public void setPageCount(int pages)
    {
        _pageCount=pages;
        initProgressBar(_pageProgress, pages);
    }

    public void setCurrentPage(int page)
    {
        int max=_pageCount;
        updateProgressBar(page, max,_pageProgress,null);
    }

    public void printError(String msg)
    {
        appendMessage(msg);
    }

    public void printWarn(String msg)
    {
        appendMessage(msg);
    }

    public void printMessage(String msg)
    {
        appendMessage(msg);
    }

    public void printVerbose(String msg)
    {
        if (_verbose)
            appendMessage(msg);
    }

    public void printDebug(String msg)
    {
        if (_debug&&_verbose)
            appendMessage(msg);
    }

    private void appendMessage(String aMsg)
    {
        final String msg=aMsg;

        if (_showMessages)
        {
            try
            {
                SwingUtilities.invokeAndWait(new Runnable()
                {
                    public void run()
                    {
                        _messages.append(msg + "\n");
                        _messages.setCaretPosition(_messages.getDocument().getLength());
                    }
                });
            }
            catch (Exception ex) {ex.printStackTrace();}
        }
    }

    public void endDocument()
    {
        updateProgressBar(_pageCount,_pageCount,_pageProgress,"finished");
    }

    public void endDocuments()
    {
        updateProgressBar(_pageCount, _pageCount, _pageProgress, "finished");
        updateProgressBar(_fileCount, _fileCount, _fileProgress, "finished");
        updateLabel(_fileName,"");
    }

    private void updateProgressBar(int currentValue, int maxValue, JProgressBar aProgressBar, String textValue)
    {
        final int value=currentValue;
        final int max=maxValue;
        final JProgressBar progressBar=aProgressBar;
        final String text=textValue;

        try
        {
            SwingUtilities.invokeAndWait(new Runnable()
            {
                public void run()
                {
                    progressBar.setValue(value);
                    if (null!=text)
                        progressBar.setString(text);
                    else
                        progressBar.setString("" + value + "/" + max);
                    System.out.println("" + value + "/" + max);
                }
            });
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    private void initProgressBar(JProgressBar aProgressBar, int maxValue)
    {
        final JProgressBar progressBar=aProgressBar;
        final int max=maxValue;
        try
        {
            SwingUtilities.invokeAndWait(new Runnable(){
                public void run()
                {
                    progressBar.setMinimum(0);
                    progressBar.setMaximum(max);
                    progressBar.setValue(0);
                    progressBar.setString("0/" + max);
                    System.out.println("0/" + max);
                }
            });
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    private void updateLabel(JLabel aLabel, String aText)
    {
        final JLabel label=aLabel;
        final String text=aText;
        try
        {
            SwingUtilities.invokeAndWait(new Runnable()
            {
                public void run()
                {
                    label.setText(text);
                    System.out.println(text);
                }
            });
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }

    public void showMessages(boolean verbose)
    {
        _messages.setVisible(verbose);
        _messagesLabel.setVisible(verbose);
        _messagesPane.setVisible(verbose);
        _showMessages=verbose;
    }
}
